@@ -1,6 +1,6 @@
 # This relies on each of the submodules having an __all__ variable.
 
-__version__ = '0.8.4'
+__version__ = '0.9.0'
 
 
 from .protocol import *
@@ -8,6 +8,7 @@ from .connector import *
 from .client import *
 from .errors import *
 from .parsers import *
+from .streams import *
 
 
 __all__ = (client.__all__ +
@@ -15,4 +16,5 @@ __all__ = (client.__all__ +
            parsers.__all__ +
            protocol.__all__ +
            connector.__all__ +
+           streams.__all__ +
            ['__version__'])
