@@ -1,9 +1,8 @@
 """HTTP Client for asyncio."""
 
-__all__ = ['request', 'HttpClient', 'BasicAuth']
+__all__ = ['request', 'HttpClient']
 
 import asyncio
-import base64
 import collections
 import http.cookies
 import json
@@ -12,21 +11,23 @@ import inspect
 import itertools
 import random
 import time
-import uuid
 import urllib.parse
 import weakref
 import warnings
+try:
+    import chardet
+except ImportError:  # pragma: no cover
+    chardet = None
 
 import aiohttp
-from . import helpers
+from . import helpers, streams
 from .log import client_log
+from .streams import EOF_MARKER, FlowControlStreamReader
 from .multidict import CaseInsensitiveMultiDict, MultiDict, MutableMultiDict
 
 HTTP_PORT = 80
 HTTPS_PORT = 443
 
-BasicAuth = collections.namedtuple('BasicAuth', ['login', 'password'])
-
 
 @asyncio.coroutine
 def request(method, url, *,
@@ -39,7 +40,7 @@ def request(method, url, *,
             allow_redirects=True,
             max_redirects=10,
             encoding='utf-8',
-            version=(1, 1),
+            version=aiohttp.HttpVersion11,
             compress=None,
             chunked=None,
             expect100=False,
@@ -59,11 +60,10 @@ def request(method, url, *,
     :param headers: (optional) Dictionary of HTTP Headers to send with
       the request
     :param cookies: (optional) Dict object to send with the request
-    :param files: (optional) Dictionary of 'name': file-like-objects
-       for multipart encoding upload
     :param auth: (optional) BasicAuth named tuple represent HTTP Basic Auth
     :param allow_redirects: (optional) Boolean. Set to True if POST/PUT/DELETE
        redirect following is allowed.
+    :param version: Request http version.
     :param compress: Boolean. Set to True if request has to be compressed
        with deflate encoding.
     :param chunked: Boolean or Integer. Set to chunk size for chunked
@@ -98,7 +98,7 @@ def request(method, url, *,
         req = request_class(
             method, url, params=params, headers=headers, data=data,
             cookies=cookies, files=files, encoding=encoding,
-            version=version, compress=compress, chunked=chunked,
+            auth=auth, version=version, compress=compress, chunked=chunked,
             loop=loop, expect100=expect100, response_class=response_class)
 
         try:
@@ -172,8 +172,9 @@ class ClientRequest:
 
     def __init__(self, method, url, *,
                  params=None, headers=None, data=None, cookies=None,
-                 files=None, auth=None, encoding='utf-8', version=(1, 1),
-                 compress=None, chunked=None, expect100=False, verify_ssl=True,
+                 files=None, auth=None, encoding='utf-8',
+                 version=aiohttp.HttpVersion11, compress=None,
+                 chunked=None, expect100=False, verify_ssl=True,
                  loop=None, response_class=None):
         self.url = url
         self.method = method.upper()
@@ -192,11 +193,18 @@ class ClientRequest:
         self.update_content_encoding()
         self.update_auth(auth)
 
-        if data and not files:
-            if self.method not in self.GET_METHODS:
-                self.update_body_from_data(data)
-        elif files:
-            self.update_body_from_files(files, data)
+        if files:
+            warnings.warn(
+                'files parameter is deprecated. use data instead',
+                DeprecationWarning)
+            if data:
+                raise ValueError(
+                    'data and files parameters are '
+                    'not supported at the same time.')
+            data = files
+
+        if data:
+            self.update_body_from_data(data)
 
         self.update_transfer_encoding()
         self.update_expect_continue(expect100)
@@ -216,11 +224,7 @@ class ClientRequest:
         # basic auth info
         if '@' in netloc:
             authinfo, netloc = netloc.split('@', 1)
-            creds = authinfo.split(':', 1)
-            if len(creds) > 1:
-                self.auth = BasicAuth(creds[0], creds[1])
-            else:
-                self.auth = BasicAuth(creds[0], '')
+            self.auth = helpers.BasicAuth(*authinfo.split(':', 1))
 
         # Record entire netloc for usage in host header
         self.netloc = netloc
@@ -345,93 +349,46 @@ class ClientRequest:
         if auth is None:
             return
 
-        if not isinstance(auth, BasicAuth):
+        if not isinstance(auth, helpers.BasicAuth):
             warnings.warn(
                 'BasicAuth() tuple is required instead ', DeprecationWarning)
+            auth = helpers.BasicAuth(*auth)
 
-        basic_login, basic_passwd = auth
-
-        if basic_login is not None and basic_passwd is not None:
-            self.headers['AUTHORIZATION'] = 'Basic %s' % (
-                base64.b64encode(
-                    ('%s:%s' % (basic_login, basic_passwd)).encode('latin1'))
-                .strip().decode('latin1'))
-        elif basic_login is not None or basic_passwd is not None:
-            raise ValueError("HTTP Auth login or password is missing")
+        self.headers['AUTHORIZATION'] = auth.encode()
 
     def update_body_from_data(self, data):
-        if (hasattr(data, '__iter__') and not isinstance(
-                data, (bytes, bytearray, str, list, dict))):
+        if isinstance(data, str):
+            data = data.encode(self.encoding)
+
+        if isinstance(data, (bytes, bytearray)):
+            self.body = data
+            if 'CONTENT-TYPE' not in self.headers:
+                self.headers['CONTENT-TYPE'] = 'application/octet-stream'
+            if 'CONTENT-LENGTH' not in self.headers and not self.chunked:
+                self.headers['CONTENT-LENGTH'] = str(len(self.body))
+
+        elif isinstance(data, (asyncio.StreamReader, streams.DataQueue)):
+            self.body = data
+
+        elif (hasattr(data, '__iter__') and not
+              isinstance(data, (tuple, list, dict, io.IOBase))):
             self.body = data
             if 'CONTENT-LENGTH' not in self.headers and self.chunked is None:
                 self.chunked = True
         else:
-            if isinstance(data, (bytes, bytearray)):
-                self.body = data
-                if 'CONTENT-TYPE' not in self.headers:
-                    self.headers['CONTENT-TYPE'] = 'application/octet-stream'
+            if not isinstance(data, helpers.FormData):
+                data = helpers.FormData(data)
+
+            self.body = data(self.encoding)
+
+            if 'CONTENT-TYPE' not in self.headers:
+                self.headers['CONTENT-TYPE'] = data.contenttype
+
+            if data.is_form_data():
+                self.chunked = self.chunked or 8196
             else:
-                # form data (x-www-form-urlencoded)
-                if isinstance(data, dict):
-                    data = list(data.items())
-
-                if not isinstance(data, str):
-                    data = urllib.parse.urlencode(data, doseq=True)
-
-                self.body = data.encode(self.encoding)
-
-                if 'CONTENT-TYPE' not in self.headers:
-                    self.headers['CONTENT-TYPE'] = (
-                        'application/x-www-form-urlencoded')
-
-            if 'CONTENT-LENGTH' not in self.headers and not self.chunked:
-                self.headers['CONTENT-LENGTH'] = str(len(self.body))
-
-    def update_body_from_files(self, files, data):
-        """Generate multipart/form-data body."""
-        fields = []
-
-        if data:
-            if not isinstance(data, (list, dict)):
-                raise NotImplementedError(
-                    'Streamed body is not compatible with files.')
-
-            if isinstance(data, dict):
-                data = data.items()
-
-            for field, val in data:
-                fields.append((field, helpers.str_to_bytes(val)))
-
-        if isinstance(files, dict):
-            files = list(files.items())
-
-        for rec in files:
-            if not isinstance(rec, (tuple, list)):
-                rec = (rec,)
-
-            ft = None
-            if len(rec) == 1:
-                k = helpers.guess_filename(rec[0], 'unknown')
-                fields.append((k, k, rec[0]))
-
-            elif len(rec) == 2:
-                k, fp = rec
-                fn = helpers.guess_filename(fp, k)
-                fields.append((k, fn, fp))
-
-            else:
-                k, fp, ft = rec
-                fn = helpers.guess_filename(fp, k)
-                fields.append((k, fn, fp, ft))
-
-        self.chunked = self.chunked or 8192
-        boundary = uuid.uuid4().hex
-
-        self.body = helpers.encode_multipart_data(
-            fields, bytes(boundary, 'latin1'))
-
-        self.headers['CONTENT-TYPE'] = (
-            'multipart/form-data; boundary=%s' % boundary)
+                if 'CONTENT-LENGTH' not in self.headers and not self.chunked:
+                    self.headers['CONTENT-LENGTH'] = str(len(self.body))
 
     def update_transfer_encoding(self):
         """Analyze transfer-encoding header."""
@@ -502,6 +459,23 @@ class ClientRequest:
                         raise ValueError(
                             'Bytes object is expected, got: %s.' %
                             type(result))
+
+            elif isinstance(self.body, asyncio.StreamReader):
+                chunk = yield from self.body.read(streams.DEFAULT_LIMIT)
+                while chunk:
+                    yield from request.write(chunk)
+                    chunk = yield from self.body.read(streams.DEFAULT_LIMIT)
+
+            elif isinstance(self.body, streams.DataQueue):
+                while True:
+                    try:
+                        chunk = yield from self.body.read()
+                        if chunk is EOF_MARKER:
+                            break
+                        yield from request.write(chunk)
+                    except streams.EofStream:
+                        break
+
             else:
                 if isinstance(self.body, (bytes, bytearray)):
                     self.body = (self.body,)
@@ -560,6 +534,7 @@ class ClientResponse:
     content = None  # Payload stream
 
     connection = None  # current connection
+    flow_control_class = FlowControlStreamReader  # reader flow control
     _reader = None     # input stream
     _response_parser = aiohttp.HttpResponseParser()
     _connection_wr = None  # weakref to self for releasing connection on del
@@ -593,6 +568,8 @@ class ClientResponse:
     def _setup_connection(self, connection):
         self._reader = connection.reader
         self.connection = connection
+        self.content = self.flow_control_class(
+            connection.reader, loop=connection.loop)
 
         msg = ('ClientResponse has to be closed explicitly! {}:{}:{}'
                .format(self.method, self.host, self.url))
@@ -631,10 +608,11 @@ class ClientResponse:
 
         # payload
         response_with_body = self.method.lower() != 'head'
-        self.content = self._reader.set_parser(
+        self._reader.set_parser(
             aiohttp.HttpPayloadParser(self.message,
                                       readall=read_until_eof,
-                                      response_with_body=response_with_body))
+                                      response_with_body=response_with_body),
+            self.content)
 
         # cookies
         self.cookies = http.cookies.SimpleCookie()
@@ -650,6 +628,9 @@ class ClientResponse:
 
     def close(self, force=False):
         if self.connection is not None:
+            if self.content and not self.content.at_eof():
+                force = True
+
             if force:
                 self.connection.close()
             else:
@@ -663,14 +644,12 @@ class ClientResponse:
 
     @asyncio.coroutine
     def release(self):
-        force = False
         try:
-            while True:
-                yield from self.content.read()
-        except Exception as exc:
-            force = not isinstance(exc, aiohttp.EofStream)
+            chunk = yield from self.content.readany()
+            while chunk is not EOF_MARKER or chunk:
+                chunk = yield from self.content.readany()
         finally:
-            self.close(force=force)
+            self.close()
 
     @asyncio.coroutine
     def wait_for_close(self):
@@ -684,29 +663,15 @@ class ClientResponse:
 
     @asyncio.coroutine
     def read(self, decode=False):
-        """Read response payload. Decode known types of content."""
+        """Read response payload."""
         if self._content is None:
-            buf = []
-            total = 0
             try:
-                while True:
-                    chunk = yield from self.content.read()
-                    size = len(chunk)
-                    buf.append((chunk, size))
-                    total += size
-            except aiohttp.EofStream:
-                self.close()
+                self._content = yield from self.content.read()
             except:
                 self.close(True)
                 raise
-
-            self._content = bytearray(total)
-
-            idx = 0
-            content = memoryview(self._content)
-            for chunk, size in buf:
-                content[idx:idx+size] = chunk
-                idx += size
+            else:
+                self.close()
 
         data = self._content
 
@@ -728,6 +693,27 @@ class ClientResponse:
         )
         return (yield from self.read(decode))
 
+    def _get_encoding(self, encoding):
+        ctype = self.headers.get('CONTENT-TYPE', '').lower()
+        mtype, stype, _, params = helpers.parse_mimetype(ctype)
+
+        if not encoding:
+            encoding = params.get('charset')
+            if not encoding and chardet:
+                encoding = chardet.detect(self._content)['encoding']
+            if not encoding:  # pragma: no cover
+                encoding = 'utf-8'
+
+        return encoding
+
+    @asyncio.coroutine
+    def text(self, encoding=None):
+        """Read response payload and decode."""
+        if self._content is None:
+            yield from self.read()
+
+        return self._content.decode(self._get_encoding(encoding))
+
     @asyncio.coroutine
     def json(self, *, encoding=None, loads=json.loads):
         """Reads and decodes JSON response."""
@@ -735,16 +721,14 @@ class ClientResponse:
             yield from self.read()
 
         ctype = self.headers.get('CONTENT-TYPE', '').lower()
-        mtype, stype, _, params = helpers.parse_mimetype(ctype)
-        if not (mtype == 'application' or stype == 'json'):
+        if 'json' not in ctype:
             client_log.warning(
                 'Attempt to decode JSON with unexpected mimetype: %s', ctype)
 
         if not self._content.strip():
             return None
 
-        encoding = encoding or params.get('charset', 'utf-8')
-        return loads(self._content.decode(encoding))
+        return loads(self._content.decode(self._get_encoding(encoding)))
 
 
 class HttpClient:
@@ -840,7 +824,7 @@ class HttpClient:
                 allow_redirects=True,
                 max_redirects=10,
                 encoding='utf-8',
-                version=(1, 1),
+                version=aiohttp.HttpVersion11,
                 compress=None,
                 chunked=None,
                 expect100=False,
