@@ -56,22 +56,18 @@ _SocketSocketTransport ->
 
 """
 __all__ = ['EofStream', 'StreamParser', 'StreamProtocol',
-           'ParserBuffer', 'DataQueue', 'LinesParser', 'ChunksParser']
+           'ParserBuffer', 'LinesParser', 'ChunksParser']
 
 import asyncio
 import asyncio.streams
-import collections
 import inspect
 from . import errors
+from .streams import FlowControlDataQueue, EofStream
 
 BUF_LIMIT = 2**14
 DEFAULT_LIMIT = 2**16
 
 
-class EofStream(Exception):
-    """eof stream indication."""
-
-
 class StreamParser:
     """StreamParser manages incoming bytes stream and protocol parsers.
 
@@ -181,12 +177,14 @@ class StreamParser:
         self._buffer.shrink()
         self._eof = True
 
-    def set_parser(self, parser):
+    def set_parser(self, parser, output=None):
         """set parser to stream. return parser's DataQueue."""
         if self._parser:
             self.unset_parser()
 
-        output = DataQueue(self, loop=self._loop)
+        if output is None:
+            output = FlowControlDataQueue(self, loop=self._loop)
+
         if self._exception:
             output.set_exception(self._exception)
             return output
@@ -278,73 +276,6 @@ class StreamProtocol(asyncio.streams.FlowControlMixin, asyncio.Protocol):
         return waiter
 
 
-class DataQueue:
-    """DataQueue is a destination for parsed data."""
-
-    def __init__(self, stream, *, loop=None):
-        self._stream = stream
-        self._loop = loop
-        self._buffer = collections.deque()
-        self._eof = False
-        self._waiter = None
-        self._exception = None
-
-    def at_eof(self):
-        return self._eof
-
-    def exception(self):
-        return self._exception
-
-    def set_exception(self, exc):
-        self._exception = exc
-
-        waiter = self._waiter
-        if waiter is not None:
-            self._waiter = None
-            if not waiter.done():
-                waiter.set_exception(exc)
-
-    def feed_data(self, data):
-        self._buffer.append(data)
-
-        waiter = self._waiter
-        if waiter is not None:
-            self._waiter = None
-            if not waiter.cancelled():
-                waiter.set_result(True)
-
-    def feed_eof(self):
-        self._eof = True
-
-        waiter = self._waiter
-        if waiter is not None:
-            self._waiter = None
-            if not waiter.cancelled():
-                waiter.set_result(False)
-
-    @asyncio.coroutine
-    def read(self):
-        self._stream.resume_stream()
-        try:
-            if not self._buffer and not self._eof:
-                if self._exception is not None:
-                    raise self._exception
-
-                assert not self._waiter
-                self._waiter = asyncio.Future(loop=self._loop)
-                yield from self._waiter
-
-            if self._buffer:
-                return self._buffer.popleft()
-            else:
-                if self._exception is not None:
-                    raise self._exception
-                else:
-                    raise EofStream
-        finally:
-            self._stream.pause_stream()
-
-
 class ParserBuffer(bytearray):
     """ParserBuffer is a bytearray extension.
 
