@@ -1,6 +1,7 @@
 """Http related parsers and protocol."""
 
 __all__ = ['HttpMessage', 'Request', 'Response',
+           'HttpVersion', 'HttpVersion10', 'HttpVersion11',
            'RawRequestMessage', 'RawResponseMessage',
            'HttpPrefixParser', 'HttpRequestParser', 'HttpResponseParser',
            'HttpPayloadParser']
@@ -28,6 +29,11 @@ EOL_MARKER = object()
 
 RESPONSES = http.server.BaseHTTPRequestHandler.responses
 
+HttpVersion = collections.namedtuple(
+    'HttpVersion', ['major', 'minor'])
+HttpVersion10 = HttpVersion(1, 0)
+HttpVersion11 = HttpVersion(1, 1)
+
 
 RawRequestMessage = collections.namedtuple(
     'RawRequestMessage',
@@ -60,7 +66,7 @@ class HttpParser:
         lines_idx = 1
         line = lines[1]
 
-        while line not in ('\r\n', '\n'):
+        while line:
             header_length = len(line)
 
             # Parse initial header name : value pair.
@@ -78,7 +84,7 @@ class HttpParser:
             line = lines[lines_idx]
 
             # consume continuation lines
-            continuation = line[0] in CONTINUATION
+            continuation = line and line[0] in CONTINUATION
 
             if continuation:
                 value = [value]
@@ -93,7 +99,7 @@ class HttpParser:
                     lines_idx += 1
                     line = lines[lines_idx]
                     continuation = line[0] in CONTINUATION
-                value = ''.join(value)
+                value = '\r\n'.join(value)
             else:
                 if header_length > self.max_field_size:
                     raise errors.LineTooLong(
@@ -156,7 +162,7 @@ class HttpRequestParser(HttpParser):
             raise errors.LineTooLong(exc.limit) from None
 
         lines = raw_data.decode(
-            'ascii', 'surrogateescape').splitlines(True)
+            'ascii', 'surrogateescape').split('\r\n')
 
         # request line
         line = lines[0]
@@ -174,11 +180,11 @@ class HttpRequestParser(HttpParser):
         match = VERSRE.match(version)
         if match is None:
             raise errors.BadStatusLine(version)
-        version = (int(match.group(1)), int(match.group(2)))
+        version = HttpVersion(int(match.group(1)), int(match.group(2)))
 
         # read headers
         headers, close, compression = self.parse_headers(lines)
-        if version <= (1, 0):
+        if version <= HttpVersion10:
             close = True
         elif close is None:
             close = False
@@ -205,7 +211,7 @@ class HttpResponseParser(HttpParser):
                 raise errors.LineTooLong(exc.limit) from None
 
             lines = raw_data.decode(
-                'ascii', 'surrogateescape').splitlines(True)
+                'ascii', 'surrogateescape').split('\r\n')
 
             line = lines[0]
             try:
@@ -222,7 +228,7 @@ class HttpResponseParser(HttpParser):
             match = VERSRE.match(version)
             if match is None:
                 raise errors.BadStatusLine(line)
-            version = (int(match.group(1)), int(match.group(2)))
+            version = HttpVersion(int(match.group(1)), int(match.group(2)))
 
             # The status code is a three-digit number
             try:
@@ -237,7 +243,7 @@ class HttpResponseParser(HttpParser):
             headers, close, compression = self.parse_headers(lines)
 
             if close is None:
-                close = version <= (1, 0)
+                close = version <= HttpVersion10
 
             out.feed_data(
                 RawResponseMessage(
@@ -253,8 +259,8 @@ class HttpResponseParser(HttpParser):
 
 class HttpPayloadParser:
 
-    def __init__(self, message, length=None, compression=True, readall=False,
-                 response_with_body=True):
+    def __init__(self, message, length=None, compression=True,
+                 readall=False, response_with_body=True):
         self.message = message
         self.length = length
         self.compression = compression
@@ -533,7 +539,7 @@ class HttpMessage:
         self.closing = close
 
         # disable keep-alive for http/1.0
-        if version <= (1, 0):
+        if version <= HttpVersion10:
             self.keepalive = False
         else:
             self.keepalive = None
@@ -582,7 +588,7 @@ class HttpMessage:
             # connection keep-alive
             elif 'close' in val:
                 self.keepalive = False
-            elif 'keep-alive' in val and self.version >= (1, 1):
+            elif 'keep-alive' in val and self.version >= HttpVersion11:
                 self.keepalive = True
 
         elif name == 'UPGRADE':
@@ -616,7 +622,7 @@ class HttpMessage:
 
         if (self.chunked is True) or (
                 self.length is None and
-                self.version >= (1, 1) and
+                self.version >= HttpVersion11 and
                 self.status not in (304, 204)):
             self.chunked = True
             self.writer = self._write_chunked_payload()
@@ -806,7 +812,8 @@ class Response(HttpMessage):
         'DATE',
     }
 
-    def __init__(self, transport, status, http_version=(1, 1), close=False):
+    def __init__(self, transport, status,
+                 http_version=HttpVersion11, close=False):
         super().__init__(transport, http_version, close)
 
         self.status = status
@@ -826,7 +833,7 @@ class Request(HttpMessage):
     HOP_HEADERS = ()
 
     def __init__(self, transport, method, path,
-                 http_version=(1, 1), close=False):
+                 http_version=HttpVersion11, close=False):
         super().__init__(transport, http_version, close)
 
         self.method = method
