@@ -12,6 +12,11 @@ import socket
 import aiohttp
 from aiohttp.client import ClientRequest, ClientResponse, HttpClient
 
+try:
+    import chardet
+except ImportError:
+    chardet = None
+
 
 class RequestTests(unittest.TestCase):
 
@@ -109,9 +114,9 @@ class ClientResponseTests(unittest.TestCase):
 
     def test_release(self):
         fut = asyncio.Future(loop=self.loop)
-        fut.set_exception(aiohttp.EofStream)
+        fut.set_result(b'')
         content = self.response.content = unittest.mock.Mock()
-        content.read.return_value = fut
+        content.readany.return_value = fut
         self.response.close = unittest.mock.Mock()
 
         self.loop.run_until_complete(self.response.release())
@@ -136,6 +141,65 @@ class ClientResponseTests(unittest.TestCase):
         self.assertEqual(res, 'json')
         self.assertTrue(self.response.json.called)
 
+    def test_text(self):
+        def side_effect(*args, **kwargs):
+            def second_call(*args, **kwargs):
+                raise aiohttp.EofStream
+            fut = asyncio.Future(loop=self.loop)
+            fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
+            content.read.side_effect = second_call
+            return fut
+        self.response.headers = {
+            'CONTENT-TYPE': 'application/json;charset=cp1251'}
+        content = self.response.content = unittest.mock.Mock()
+        content.read.side_effect = side_effect
+        self.response.close = unittest.mock.Mock()
+
+        res = self.loop.run_until_complete(self.response.text())
+        self.assertEqual(res, '{"тест": "пройден"}')
+        self.assertTrue(self.response.close.called)
+
+    def test_text_custom_encoding(self):
+        def side_effect(*args, **kwargs):
+            def second_call(*args, **kwargs):
+                raise aiohttp.EofStream
+            fut = asyncio.Future(loop=self.loop)
+            fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
+            content.read.side_effect = second_call
+            return fut
+        self.response.headers = {
+            'CONTENT-TYPE': 'application/json'}
+        content = self.response.content = unittest.mock.Mock()
+        content.read.side_effect = side_effect
+        self.response.close = unittest.mock.Mock()
+
+        res = self.loop.run_until_complete(
+            self.response.text(encoding='cp1251'))
+        self.assertEqual(res, '{"тест": "пройден"}')
+        self.assertTrue(self.response.close.called)
+
+    def test_text_detect_encoding(self):
+        def side_effect(*args, **kwargs):
+            def second_call(*args, **kwargs):
+                raise aiohttp.EofStream
+            fut = asyncio.Future(loop=self.loop)
+            fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
+            content.read.side_effect = second_call
+            return fut
+        self.response.headers = {'CONTENT-TYPE': 'application/json'}
+        content = self.response.content = unittest.mock.Mock()
+        content.read.side_effect = side_effect
+        self.response.close = unittest.mock.Mock()
+
+        if chardet is None:
+            self.assertRaises(UnicodeDecodeError,
+                              self.loop.run_until_complete,
+                              self.response.text())
+        else:
+            res = self.loop.run_until_complete(self.response.text())
+            self.assertEqual(res, '{"тест": "пройден"}')
+            self.assertTrue(self.response.close.called)
+
     def test_json(self):
         def side_effect(*args, **kwargs):
             def second_call(*args, **kwargs):
@@ -193,6 +257,35 @@ class ClientResponseTests(unittest.TestCase):
         self.assertEqual(res, {'тест': 'пройден'})
         self.assertTrue(self.response.close.called)
 
+    def test_json_detect_encoding(self):
+        def side_effect(*args, **kwargs):
+            def second_call(*args, **kwargs):
+                raise aiohttp.EofStream
+            fut = asyncio.Future(loop=self.loop)
+            fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
+            content.read.side_effect = second_call
+            return fut
+        self.response.headers = {'CONTENT-TYPE': 'application/json'}
+        content = self.response.content = unittest.mock.Mock()
+        content.read.side_effect = side_effect
+        self.response.close = unittest.mock.Mock()
+
+        if chardet is None:
+            self.assertRaises(UnicodeDecodeError,
+                              self.loop.run_until_complete,
+                              self.response.json())
+        else:
+            res = self.loop.run_until_complete(self.response.json())
+            self.assertEqual(res, {'тест': 'пройден'})
+            self.assertTrue(self.response.close.called)
+
+    def test_override_flow_control(self):
+        class MyResponse(ClientResponse):
+            flow_control_class = aiohttp.FlowControlDataQueue
+        response = MyResponse('get', 'http://python.org')
+        response._setup_connection(self.connection)
+        self.assertIsInstance(response.content, aiohttp.FlowControlDataQueue)
+
 
 class ClientRequestTests(unittest.TestCase):
 
@@ -298,10 +391,18 @@ class ClientRequestTests(unittest.TestCase):
 
     def test_basic_auth(self):
         req = ClientRequest('get', 'http://python.org',
-                            auth=aiohttp.BasicAuth('nkim', '1234'))
+                            auth=aiohttp.helpers.BasicAuth('nkim', '1234'))
         self.assertIn('AUTHORIZATION', req.headers)
         self.assertEqual('Basic bmtpbToxMjM0', req.headers['AUTHORIZATION'])
 
+    def test_basic_auth_utf8(self):
+        req = ClientRequest('get', 'http://python.org',
+                            auth=aiohttp.helpers.BasicAuth('nkim', 'секрет',
+                                                           'utf-8'))
+        self.assertIn('AUTHORIZATION', req.headers)
+        self.assertEqual('Basic bmtpbTrRgdC10LrRgNC10YI=',
+                         req.headers['AUTHORIZATION'])
+
     def test_basic_auth_tuple_deprecated(self):
         req = ClientRequest('get', 'http://python.org', auth=('nkim', '1234'))
         self.assertIn('AUTHORIZATION', req.headers)
@@ -314,16 +415,10 @@ class ClientRequestTests(unittest.TestCase):
 
         req = ClientRequest(
             'get', 'http://nkim@python.org',
-            auth=aiohttp.BasicAuth('nkim', '1234'))
+            auth=aiohttp.helpers.BasicAuth('nkim', '1234'))
         self.assertIn('AUTHORIZATION', req.headers)
         self.assertEqual('Basic bmtpbToxMjM0', req.headers['AUTHORIZATION'])
 
-    def test_basic_auth_err(self):
-        # missing password here
-        self.assertRaises(
-            ValueError, ClientRequest,
-            'get', 'http://python.org', auth=aiohttp.BasicAuth('nkim', None))
-
     def test_no_content_length(self):
         req = ClientRequest('get', 'http://python.org', loop=self.loop)
         req.send(self.transport, self.protocol)
@@ -418,7 +513,7 @@ class ClientRequestTests(unittest.TestCase):
 
     def test_files_and_bytes_data(self):
         self.assertRaises(
-            NotImplementedError, ClientRequest,
+            ValueError, ClientRequest,
             'POST', 'http://python.org/',
             data=b'binary data', files={'file': b'file data'})
 
