@@ -150,6 +150,8 @@ class HttpClientFunctionalTests(unittest.TestCase):
 
                 self.loop.run_until_complete(go())
 
+        connector.close()
+
     def test_use_global_loop(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
             try:
@@ -279,21 +281,48 @@ class HttpClientFunctionalTests(unittest.TestCase):
             with open(__file__) as f:
                 r = self.loop.run_until_complete(
                     client.request(
-                        'post', url, files={'some': f}, chunked=1024,
+                        'post', url, data={'some': f, 'test': b'data'},
+                        chunked=1024,
                         headers={'Transfer-Encoding': 'chunked'},
                         loop=self.loop))
                 content = self.loop.run_until_complete(r.json())
+                files = list(
+                    sorted(content['multipart-data'],
+                           key=lambda d: d['name']))
 
                 f.seek(0)
                 filename = os.path.split(f.name)[-1]
 
-                self.assertEqual(1, len(content['multipart-data']))
-                self.assertEqual(
-                    'some', content['multipart-data'][0]['name'])
-                self.assertEqual(
-                    filename, content['multipart-data'][0]['filename'])
-                self.assertEqual(
-                    f.read(), content['multipart-data'][0]['data'])
+                self.assertEqual(2, len(content['multipart-data']))
+                self.assertEqual('some', files[0]['name'])
+                self.assertEqual(filename, files[0]['filename'])
+                self.assertEqual(f.read(), files[0]['data'])
+                self.assertEqual(r.status, 200)
+                r.close()
+
+    def test_POST_FILES_DEPRECATED(self):
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            url = httpd.url('method', 'post')
+
+            with open(__file__) as f:
+                r = self.loop.run_until_complete(
+                    client.request(
+                        'post', url, files={'some': f, 'test': b'data'},
+                        chunked=1024,
+                        headers={'Transfer-Encoding': 'chunked'},
+                        loop=self.loop))
+                content = self.loop.run_until_complete(r.json())
+                files = list(
+                    sorted(content['multipart-data'],
+                           key=lambda d: d['name']))
+
+                f.seek(0)
+                filename = os.path.split(f.name)[-1]
+
+                self.assertEqual(2, len(content['multipart-data']))
+                self.assertEqual('some', files[0]['name'])
+                self.assertEqual(filename, files[0]['filename'])
+                self.assertEqual(f.read(), files[0]['data'])
                 self.assertEqual(r.status, 200)
                 r.close()
 
@@ -303,7 +332,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
 
             with open(__file__) as f:
                 r = self.loop.run_until_complete(
-                    client.request('post', url, files={'some': f},
+                    client.request('post', url, data={'some': f},
                                    chunked=1024, compress='deflate',
                                    loop=self.loop))
 
@@ -329,19 +358,30 @@ class HttpClientFunctionalTests(unittest.TestCase):
 
             with open(__file__) as f:
                 r = self.loop.run_until_complete(
-                    client.request('post', url, files=[('some', f.read())],
+                    client.request('post', url, data=[('some', f.read())],
                                    loop=self.loop))
 
                 content = self.loop.run_until_complete(r.json())
 
                 f.seek(0)
-                self.assertEqual(1, len(content['multipart-data']))
-                self.assertEqual(
-                    'some', content['multipart-data'][0]['name'])
-                self.assertEqual(
-                    'some', content['multipart-data'][0]['filename'])
-                self.assertEqual(
-                    f.read(), content['multipart-data'][0]['data'])
+                self.assertEqual(1, len(content['form']))
+                self.assertIn('some', content['form'])
+                self.assertEqual(f.read(), content['form']['some'][0])
+                self.assertEqual(r.status, 200)
+                r.close()
+
+    def test_POST_FILES_STR_SIMPLE(self):
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            url = httpd.url('method', 'post')
+
+            with open(__file__) as f:
+                r = self.loop.run_until_complete(
+                    client.request('post', url, data=f.read(), loop=self.loop))
+
+                content = self.loop.run_until_complete(r.json())
+
+                f.seek(0)
+                self.assertEqual(f.read(), content['content'])
                 self.assertEqual(r.status, 200)
                 r.close()
 
@@ -351,7 +391,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
 
             with open(__file__) as f:
                 r = self.loop.run_until_complete(
-                    client.request('post', url, files=[('some', f)],
+                    client.request('post', url, data=[('some', f)],
                                    loop=self.loop))
 
                 content = self.loop.run_until_complete(r.json())
@@ -376,7 +416,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
             with open(__file__) as f:
                 r = self.loop.run_until_complete(
                     client.request('post', url, loop=self.loop,
-                                   files=[('some', f, 'text/plain')]))
+                                   data=[('some', f, 'text/plain')]))
 
                 content = self.loop.run_until_complete(r.json())
 
@@ -401,7 +441,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
 
             with open(__file__) as f:
                 r = self.loop.run_until_complete(
-                    client.request('post', url, files=[f], loop=self.loop))
+                    client.request('post', url, data=f, loop=self.loop))
 
                 content = self.loop.run_until_complete(r.json())
 
@@ -425,19 +465,45 @@ class HttpClientFunctionalTests(unittest.TestCase):
             data = io.BytesIO(b'data')
 
             r = self.loop.run_until_complete(
-                client.request('post', url, files=[data], loop=self.loop))
+                client.request('post', url, data=[data], loop=self.loop))
 
             content = self.loop.run_until_complete(r.json())
 
             self.assertEqual(1, len(content['multipart-data']))
             self.assertEqual(
-                {'content-type': 'application/octet-stream',
+                {'content-type': 'text/plain',
                  'data': 'data',
                  'filename': 'unknown',
                  'name': 'unknown'}, content['multipart-data'][0])
             self.assertEqual(r.status, 200)
             r.close()
 
+    def test_POST_FILES_IO_WITH_PARAMS(self):
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            url = httpd.url('method', 'post')
+
+            data = io.BytesIO(b'data')
+
+            r = self.loop.run_until_complete(
+                client.request('post', url,
+                               data=(('test', 'true'), (data,)),
+                               loop=self.loop))
+
+            content = self.loop.run_until_complete(r.json())
+
+            self.assertEqual(2, len(content['multipart-data']))
+            self.assertEqual(
+                {'content-type': 'text/plain',
+                 'data': 'true',
+                 'name': 'test'}, content['multipart-data'][0])
+            self.assertEqual(
+                {'content-type': 'text/plain',
+                 'data': 'data',
+                 'filename': 'unknown',
+                 'name': 'unknown'}, content['multipart-data'][1])
+            self.assertEqual(r.status, 200)
+            r.close()
+
     def test_POST_FILES_WITH_DATA(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
             url = httpd.url('method', 'post')
@@ -445,24 +511,22 @@ class HttpClientFunctionalTests(unittest.TestCase):
             with open(__file__) as f:
                 r = self.loop.run_until_complete(
                     client.request('post', url, loop=self.loop,
-                                   data={'test': 'true'}, files={'some': f}))
+                                   data={'test': 'true', 'some': f}))
 
                 content = self.loop.run_until_complete(r.json())
+                files = list(
+                    sorted(content['multipart-data'],
+                           key=lambda d: d['name']))
 
                 self.assertEqual(2, len(content['multipart-data']))
-                self.assertEqual(
-                    'test', content['multipart-data'][0]['name'])
-                self.assertEqual(
-                    'true', content['multipart-data'][0]['data'])
+                self.assertEqual('test', files[1]['name'])
+                self.assertEqual('true', files[1]['data'])
 
                 f.seek(0)
                 filename = os.path.split(f.name)[-1]
-                self.assertEqual(
-                    'some', content['multipart-data'][1]['name'])
-                self.assertEqual(
-                    filename, content['multipart-data'][1]['filename'])
-                self.assertEqual(
-                    f.read(), content['multipart-data'][1]['data'])
+                self.assertEqual('some', files[0]['name'])
+                self.assertEqual(filename, files[0]['filename'])
+                self.assertEqual(f.read(), files[0]['data'])
                 self.assertEqual(r.status, 200)
                 r.close()
 
@@ -493,6 +557,74 @@ class HttpClientFunctionalTests(unittest.TestCase):
             self.assertEqual(str(len(data)),
                              content['headers']['Content-Length'])
 
+    def test_POST_StreamReader(self):
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            url = httpd.url('method', 'post')
+
+            with open(__file__, 'rb') as f:
+                data = f.read()
+
+            stream = aiohttp.StreamReader(loop=self.loop)
+            stream.feed_data(data)
+            stream.feed_eof()
+
+            r = self.loop.run_until_complete(
+                client.request(
+                    'post', url, data=stream,
+                    headers={'Content-Length': str(len(data))},
+                    loop=self.loop))
+            content = self.loop.run_until_complete(r.json())
+            r.close()
+
+            self.assertEqual(str(len(data)),
+                             content['headers']['Content-Length'])
+
+    def test_POST_DataQueue(self):
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            url = httpd.url('method', 'post')
+
+            with open(__file__, 'rb') as f:
+                data = f.read()
+
+            stream = aiohttp.DataQueue(loop=self.loop)
+            stream.feed_data(data[:100])
+            stream.feed_data(data[100:])
+            stream.feed_eof()
+
+            r = self.loop.run_until_complete(
+                client.request(
+                    'post', url, data=stream,
+                    headers={'Content-Length': str(len(data))},
+                    loop=self.loop))
+            content = self.loop.run_until_complete(r.json())
+            r.close()
+
+            self.assertEqual(str(len(data)),
+                             content['headers']['Content-Length'])
+
+    def test_POST_ChunksQueue(self):
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            url = httpd.url('method', 'post')
+
+            with open(__file__, 'rb') as f:
+                data = f.read()
+
+            stream = aiohttp.ChunksQueue(loop=self.loop)
+            stream.feed_data(data[:100])
+            stream.feed_data(data[100:])
+            stream.feed_eof()
+
+            r = self.loop.run_until_complete(
+                client.request(
+                    'post', url, data=stream,
+                    headers={'Content-Length': str(len(data))},
+                    loop=self.loop))
+            content = self.loop.run_until_complete(r.json())
+            r.close()
+
+            self.assertEqual(str(len(data)),
+                             content['headers']['Content-Length'])
+
     def test_expect_continue(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
             url = httpd.url('method', 'post')
@@ -650,8 +782,10 @@ class HttpClientFunctionalTests(unittest.TestCase):
             cookies = sorted([(k, v.value) for k, v in conn.cookies.items()])
             self.assertEqual(
                 cookies, [('c1', 'cookie1'), ('c2', 'cookie2'), ('test', '1')])
-        m_log.warning.assert_called_with('Can not load response cookies: %s',
-                                         mock.ANY)
+
+        m_log.warning.assert_called_with(
+            'Can not load response cookies: %s', mock.ANY)
+        conn.close()
 
     def test_multidict_headers(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
