@@ -25,18 +25,19 @@ class HttpConnectionTests(unittest.TestCase):
         self.request = mock.Mock()
         self.transport = mock.Mock()
         self.protocol = mock.Mock()
+        self.loop = mock.Mock()
 
     def test_del(self):
         conn = Connection(
             self.connector, self.key, self.request,
-            self.transport, self.protocol)
+            self.transport, self.protocol, self.loop)
         del conn
         self.assertTrue(self.transport.close.called)
 
     def test_close(self):
         conn = Connection(
             self.connector, self.key, self.request,
-            self.transport, self.protocol)
+            self.transport, self.protocol, self.loop)
         conn.close()
         self.assertTrue(self.transport.close.called)
         self.assertIsNone(conn._transport)
@@ -44,7 +45,7 @@ class HttpConnectionTests(unittest.TestCase):
     def test_release(self):
         conn = Connection(
             self.connector, self.key, self.request,
-            self.transport, self.protocol)
+            self.transport, self.protocol, self.loop)
         conn.release()
         self.assertFalse(self.transport.close.called)
         self.assertIsNone(conn._transport)
@@ -351,32 +352,13 @@ class ProxyConnectorTests(unittest.TestCase):
             loop=loop_mock)
 
     def test_proxy_auth(self):
-        with self.assertRaisesRegex(AssertionError,
-                                    "must be None or BasicAuth"):
+        with self.assertRaises(AssertionError) as ctx:
             aiohttp.ProxyConnector('http://proxy.example.com',
                                    proxy_auth=('user', 'pass'),
                                    loop=unittest.mock.Mock())
-
-    @unittest.mock.patch('aiohttp.connector.ClientRequest')
-    def test_proxy_override_auth(self, ClientRequestMock):
-        req = ClientRequest('GET', 'http://www.python.org')
-
-        loop_mock = unittest.mock.Mock()
-        tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
-        self._fake_coroutine(loop_mock.create_connection, (tr, proto))
-
-        connector = aiohttp.ProxyConnector(
-            'http://user:pass@proxy.example.com',
-            proxy_auth=aiohttp.BasicAuth(None, None),
-            loop=loop_mock)
-
-        def check_proxy_req(*args, **kw):
-            proxy_req = ClientRequest(*args, **kw)
-            self.assertNotIn('AUTHORIZATION', proxy_req.headers)
-            return proxy_req
-
-        ClientRequestMock.side_effect = check_proxy_req
-        self.loop.run_until_complete(connector._create_connection(req))
+        self.assertEqual(ctx.exception.args[0],
+                         ("proxy_auth must be None or BasicAuth() tuple",
+                          ('user', 'pass')))
 
     def test_proxy_connection_error(self):
         connector = aiohttp.ProxyConnector('http://proxy.example.com',
@@ -394,7 +376,8 @@ class ProxyConnectorTests(unittest.TestCase):
     @unittest.mock.patch('aiohttp.connector.ClientRequest')
     def test_auth(self, ClientRequestMock):
         proxy_req = ClientRequest('GET', 'http://proxy.example.com',
-                                  auth=aiohttp.BasicAuth('user', 'pass'))
+                                  auth=aiohttp.helpers.BasicAuth('user',
+                                                                 'pass'))
         ClientRequestMock.return_value = proxy_req
         self.assertIn('AUTHORIZATION', proxy_req.headers)
         self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
@@ -402,7 +385,7 @@ class ProxyConnectorTests(unittest.TestCase):
         loop_mock = unittest.mock.Mock()
         connector = aiohttp.ProxyConnector(
             'http://proxy.example.com', loop=loop_mock,
-            proxy_auth=aiohttp.BasicAuth('user', 'pass'))
+            proxy_auth=aiohttp.helpers.BasicAuth('user', 'pass'))
         connector._resolve_host = resolve_mock = unittest.mock.Mock()
         self._fake_coroutine(resolve_mock, [unittest.mock.MagicMock()])
 
@@ -422,9 +405,15 @@ class ProxyConnectorTests(unittest.TestCase):
 
         ClientRequestMock.assert_called_with(
             'GET', 'http://proxy.example.com',
-            auth=aiohttp.BasicAuth('user', 'pass'),
+            auth=aiohttp.helpers.BasicAuth('user', 'pass'),
             loop=unittest.mock.ANY, headers=unittest.mock.ANY)
 
+    def test_auth_utf8(self):
+        proxy_req = ClientRequest(
+            'GET', 'http://proxy.example.com',
+            auth=aiohttp.helpers.BasicAuth('юзер', 'пасс', 'utf-8'))
+        self.assertIn('AUTHORIZATION', proxy_req.headers)
+
     @unittest.mock.patch('aiohttp.connector.ClientRequest')
     def test_auth_from_url(self, ClientRequestMock):
         proxy_req = ClientRequest('GET', 'http://user:pass@proxy.example.com')
