@@ -36,3 +36,19 @@ class HelpersTests(unittest.TestCase):
         self.assertEqual(
             helpers.parse_mimetype('text/plain;base64'),
             ('text', 'plain', '', {'base64': ''}))
+
+    def test_basic_auth(self):
+        # missing password here
+        self.assertRaises(
+            ValueError, helpers.BasicAuth, None)
+        self.assertRaises(
+            ValueError, helpers.BasicAuth, 'nkim', None)
+
+        auth = helpers.BasicAuth('nkim')
+        self.assertEqual(auth.login, 'nkim')
+        self.assertEqual(auth.password, '')
+
+        auth = helpers.BasicAuth('nkim', 'pwd')
+        self.assertEqual(auth.login, 'nkim')
+        self.assertEqual(auth.password, 'pwd')
+        self.assertEqual(auth.encode(), 'Basic bmtpbTpwd2Q=')
