@@ -186,6 +186,26 @@ class HttpServerProtocolTests(unittest.TestCase):
         self.assertIn(b'X-SERVER: asyncio', content)
         self.assertFalse(srv._keep_alive)
 
+    def test_handle_error__utf(self):
+        transport = unittest.mock.Mock()
+        srv = server.ServerHttpProtocol(debug=True, loop=self.loop)
+        srv.connection_made(transport)
+        srv.keep_alive(True)
+        srv.writer = unittest.mock.Mock()
+
+        try:
+            raise RuntimeError('что-то пошло не так')
+        except RuntimeError as exc:
+            srv.handle_error(exc=exc)
+        content = b''.join(
+            [c[1][0] for c in list(srv.writer.write.mock_calls)])
+        self.assertIn(b'HTTP/1.1 500 Internal Server Error', content)
+        self.assertIn(b'CONTENT-TYPE: text/html; charset=utf-8', content)
+        self.assertIn(
+            "raise RuntimeError('что-то пошло не так')".encode('utf-8'),
+            content)
+        self.assertFalse(srv._keep_alive)
+
     @unittest.mock.patch('aiohttp.server.traceback')
     def test_handle_error_traceback_exc(self, m_trace):
         transport = unittest.mock.Mock()
@@ -247,6 +267,23 @@ class HttpServerProtocolTests(unittest.TestCase):
         self.assertTrue(handle.called)
         self.assertTrue(transport.close.called)
 
+    def test_handle_uncompleted(self):
+        transport = unittest.mock.Mock()
+        srv = server.ServerHttpProtocol(loop=self.loop)
+        srv.connection_made(transport)
+
+        handle = srv.handle_request = unittest.mock.Mock()
+        handle.side_effect = ValueError
+
+        srv.reader.feed_data(
+            b'GET / HTTP/1.0\r\n'
+            b'Host: example.com\r\n'
+            b'Content-Length: 50000\r\n\r\n')
+
+        self.loop.run_until_complete(srv._request_handler)
+        self.assertTrue(handle.called)
+        self.assertTrue(transport.close.called)
+
     def test_handle_coro(self):
         transport = unittest.mock.Mock()
         srv = server.ServerHttpProtocol(loop=self.loop)
